#!/bin/bash


###################################################################################################################
#
# Script Name    : traffic.sh
# Description    : This script calculate the bandwidth usage (in/out) of each interface From Fortigate via SNMPv2.
#                  It is probably compatible with any equipments using the standard MIB from ISO/CCITT.
# Author         : https://github.com/choupit0
# Site           : https://hack2know.how/
# Date           : 20210111
# Version        : 1.0.0
# Usage          : (chmod +x traffic.sh) ./traffic.sh [host]
# Prerequiste    : Configure your FortiGate to get status information via SNMP:
#                  https://kb.fortinet.com/kb/documentLink.do?externalID=FD45755
#
###################################################################################################################


# Parameters to set before to launch the script
host="$1"
community="SNMPv2 community name"

# No paramaters
if [[ "$1" == "" ]]; then
	echo "Host to check?"
	exit 1
fi

# Cleaning
rm -f interfaces.txt ref_ifHCInOctets.txt ref_ifHCOutOctets.txt new_ifHCInOctets.txt new_ifHCOutOctets.txt bandwidth.txt ifName.txt 2>/dev/null

# Getting interface name information via SNMP
snmpwalk -v 2c -c ${community} ${host} IF-MIB::ifName > ifName.txt

# Filtering the file
awk -F" " '{print $1 " " $4}' ifName.txt | cut -d"." -f2 > interfaces.txt

# Function to get interfaces counters
get_counters(){
snmpwalk -Oq -v 2c -c ${community} ${host} IF-MIB::ifHCInOctets | cut -d"." -f2 > ${1}ifHCInOctets.txt &
snmpwalk -Oq -v 2c -c ${community} ${host} IF-MIB::ifHCOutOctets | cut -d"." -f2 > ${1}ifHCOutOctets.txt &
}

# We run the function twice in order to have reference values
get_counters "ref_"
wait
sleep 5
get_counters "new_"
wait

# We calculate the bandwidth usage (in/out) of each interface (Kbits)
# We are using 'bc' instead 'expr' binary to avoid the error below:
# expr: 18446612137261418632: Numerical result out of range
# The error appear when 'expr' is not using the GNU Multi Precision library (not compiled)
# Instead it is using the normal C math operations, limited by the max integer & real number sizes
cat interfaces.txt | while read line; do
        id="$(echo $line | cut -d" " -f1)"
        name="$(echo $line | cut -d" " -f2)"
        ref_in="$(grep "^${id} " ref_ifHCInOctets.txt | cut -d" " -f2)"
        ref_out="$(grep "^${id} " ref_ifHCOutOctets.txt | cut -d" " -f2)"
        new_in="$(grep "^${id} " new_ifHCInOctets.txt | cut -d" " -f2)"
        new_out="$(grep "^${id} " new_ifHCOutOctets.txt | cut -d" " -f2)"

        # Steps (5 seconds):
        # 1) Transferred Bytes/Octets in 5 seconds
        # 2) Transferred Bytes/Octets in 1 second
        # 3) Convert to bits per second
        # 4) Convert to Kbits per second
        # Replace /1024 by /1048576 to convert to Mbits per second

        in=$(echo $( echo "((${new_in} - ${ref_in}) / 5) * 8 / 1024" | bc))
        out=$(echo $( echo "((${new_out} - ${ref_out}) / 5) * 8 / 1024" | bc))

        echo -n "${name}" >> bandwidth.txt
        echo -e -n '\t' >> bandwidth.txt
        echo -n "${in}" >> bandwidth.txt
        echo -e -n '\t' >> bandwidth.txt
        echo -e -n "${out} \n" >> bandwidth.txt
done

cat bandwidth.txt

rm -f interfaces.txt ref_ifHCInOctets.txt ref_ifHCOutOctets.txt new_ifHCInOctets.txt new_ifHCOutOctets.txt bandwidth.txt ifName.txt 2>/dev/null

exit 0
